@@ -95,7 +95,8 @@ def expand_dirs(items, exclude=lambda x: x.endswith('.so')):
     return items
 
 
-def do_sign(app_dir):
+
+def do_sign(app_dir: str) -> None:
     with current_dir(join(app_dir, 'Contents')):
         # Sign all .so files
         so_files = {x for x in files_in('.') if x.endswith('.so')}
@@ -109,6 +110,10 @@ def do_sign(app_dir):
         # Sign kitten
         with current_dir('MacOS'):
             codesign('kitten')
+        # Sign sub-apps
+        for x in os.listdir('.'):
+            if x.endswith('.app'):
+                codesign(x)
 
     # Now sign the main app
     codesign(app_dir)
@@ -130,7 +135,7 @@ def sign_app(app_dir, notarize):
     with make_certificate_useable():
         do_sign(app_dir)
         if notarize:
-            notarize_app(app_dir)
+            notarize_app(app_dir, 'kitty')
 
 
 class Freeze(object):
@@ -168,6 +173,7 @@ class Freeze(object):
         self.freeze_python()
         self.add_ca_certs()
         self.build_frozen_tools()
+        self.complete_sub_bundles()
         if not self.dont_strip:
             self.strip_files()
         if not self.skip_tests:
@@ -178,6 +184,16 @@ class Freeze(object):
 
         return ret
 
+    @flush
+    def complete_sub_bundles(self):
+        count = 0
+        for qapp in glob.glob(join(self.contents_dir, '*.app')):
+            for exe in glob.glob(join(self.contents_dir, 'MacOS', '*')):
+                os.symlink(f'../../../MacOS/{os.path.basename(exe)}', os.path.join(qapp, 'Contents', 'MacOS', os.path.basename(exe)))
+                count += 1
+        if count < 2:
+            raise SystemExit(f'Could not complete sub-bundles in {self.contents_dir}')
+
     @flush
     def add_ca_certs(self):
         print('\nDownloading CA certs...')
@@ -292,9 +308,9 @@ class Freeze(object):
                 'harfbuzz.0',
                 'png16.16',
                 'lcms2.2',
-                'crypto.1.1',
-                'ssl.1.1',
-                'rsync.2',
+                'crypto.3',
+                'ssl.3',
+                'xxhash.0',
         ):
             print('\nAdding', x)
             x = f'lib{x}.dylib'
@@ -371,6 +387,9 @@ class Freeze(object):
         os.rename(join(dirname(self.contents_dir), 'bin', 'kitty'), join(self.contents_dir, 'MacOS', 'kitty'))
         shutil.rmtree(join(dirname(self.contents_dir), 'bin'))
         self.fix_dependencies_in_lib(join(self.contents_dir, 'MacOS', 'kitty'))
+        for f in glob.glob(join(self.contents_dir, '*.app', 'Contents', 'MacOS', '*')):
+            if not os.path.islink(f):
+                self.fix_dependencies_in_lib(f)
         for f in walk(pdir):
             if f.endswith('.so') or f.endswith('.dylib'):
                 self.fix_dependencies_in_lib(f)
@@ -441,7 +460,7 @@ class Freeze(object):
         py_compile(join(self.resources_dir, 'Python'))
 
     @flush
-    def makedmg(self, d, volname, format='ULFO'):
+    def makedmg(self, d, volname, format='ULMO'):
         ''' Copy a directory d into a dmg named volname '''
         print('\nMaking dmg...')
         sys.stdout.flush()
