@@ -32,10 +32,10 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <math.h>
+#include <assert.h>
 
 #include <IOKit/graphics/IOGraphicsLib.h>
 #include <CoreVideo/CVBase.h>
-#include <CoreVideo/CVDisplayLink.h>
 #include <ApplicationServices/ApplicationServices.h>
 
 
@@ -323,54 +323,7 @@ static double getFallbackRefreshRate(CGDirectDisplayID displayID)
 //////                       GLFW internal API                      //////
 //////////////////////////////////////////////////////////////////////////
 
-void _glfwClearDisplayLinks() {
-    for (size_t i = 0; i < _glfw.ns.displayLinks.count; i++) {
-        if (_glfw.ns.displayLinks.entries[i].displayLink) {
-            CVDisplayLinkStop(_glfw.ns.displayLinks.entries[i].displayLink);
-            CVDisplayLinkRelease(_glfw.ns.displayLinks.entries[i].displayLink);
-        }
-    }
-    memset(_glfw.ns.displayLinks.entries, 0, sizeof(_GLFWDisplayLinkNS) * _glfw.ns.displayLinks.count);
-    _glfw.ns.displayLinks.count = 0;
-}
-
-static CVReturn displayLinkCallback(
-        CVDisplayLinkRef displayLink UNUSED,
-        const CVTimeStamp* now UNUSED, const CVTimeStamp* outputTime UNUSED,
-        CVOptionFlags flagsIn UNUSED, CVOptionFlags* flagsOut UNUSED, void* userInfo)
-{
-    CGDirectDisplayID displayID = (uintptr_t)userInfo;
-    NSNumber *arg = [NSNumber numberWithUnsignedInt:displayID];
-    [NSApp performSelectorOnMainThread:@selector(render_frame_received:) withObject:arg waitUntilDone:NO];
-    [arg release];
-    return kCVReturnSuccess;
-}
-
-void
-_glfw_create_cv_display_link(_GLFWDisplayLinkNS *entry) {
-    CVDisplayLinkCreateWithCGDisplay(entry->displayID, &entry->displayLink);
-    CVDisplayLinkSetOutputCallback(entry->displayLink, &displayLinkCallback, (void*)(uintptr_t)entry->displayID);
-}
-
-_GLFWDisplayLinkNS*
-_glfw_create_display_link(CGDirectDisplayID displayID) {
-    if (_glfw.ns.displayLinks.count >= arraysz(_glfw.ns.displayLinks.entries) - 1) {
-        _glfwInputError(GLFW_PLATFORM_ERROR, "Too many monitors cannot create display link");
-        return NULL;
-    }
-    for (size_t i = 0; i < _glfw.ns.displayLinks.count; i++) {
-        // already created in this run
-        if (_glfw.ns.displayLinks.entries[i].displayID == displayID) return _glfw.ns.displayLinks.entries + i;
-    }
-    _GLFWDisplayLinkNS *entry = &_glfw.ns.displayLinks.entries[_glfw.ns.displayLinks.count++];
-    memset(entry, 0, sizeof(_GLFWDisplayLinkNS));
-    entry->displayID = displayID;
-    _glfw_create_cv_display_link(entry);
-    return entry;
-}
-
 // Poll for changes in the set of connected monitors
-//
 void _glfwPollMonitorsNS(void)
 {
     uint32_t displayCount;
@@ -426,7 +379,7 @@ void _glfwPollMonitorsNS(void)
             {
                 disconnected[j]->ns.displayID  = displays[i];
                 disconnected[j]->ns.screen = screen;
-                _glfw_create_display_link(displays[i]);
+                _glfwCreateDisplayLink(displays[i]);
                 disconnected[j] = NULL;
                 break;
             }
@@ -447,7 +400,7 @@ void _glfwPollMonitorsNS(void)
         monitor->ns.displayID  = displays[i];
         monitor->ns.unitNumber = unitNumber;
         monitor->ns.screen     = screen;
-        _glfw_create_display_link(monitor->ns.displayID);
+        _glfwCreateDisplayLink(monitor->ns.displayID);
 
         free(name);
 
@@ -618,11 +571,16 @@ GLFWvidmode* _glfwPlatformGetVideoModes(_GLFWmonitor* monitor, int* count)
     return result;
 }
 
-void _glfwPlatformGetVideoMode(_GLFWmonitor* monitor, GLFWvidmode *mode)
+bool _glfwPlatformGetVideoMode(_GLFWmonitor* monitor, GLFWvidmode *mode)
 {
     CGDisplayModeRef native = CGDisplayCopyDisplayMode(monitor->ns.displayID);
+    if (!native) {
+        _glfwInputError(GLFW_PLATFORM_ERROR, "Cocoa: Failed to query display mode");
+        return false;
+    }
     *mode = vidmodeFromCGDisplayMode(native, monitor->ns.fallbackRefreshRate);
     CGDisplayModeRelease(native);
+    return true;
 }
 
 bool _glfwPlatformGetGammaRamp(_GLFWmonitor* monitor, GLFWgammaramp* ramp)
@@ -678,6 +636,8 @@ void _glfwPlatformSetGammaRamp(_GLFWmonitor* monitor, const GLFWgammaramp* ramp)
 GLFWAPI CGDirectDisplayID glfwGetCocoaMonitor(GLFWmonitor* handle)
 {
     _GLFWmonitor* monitor = (_GLFWmonitor*) handle;
+    assert(monitor != NULL);
+
     _GLFW_REQUIRE_INIT_OR_RETURN(kCGNullDirectDisplay);
     return monitor->ns.displayID;
 }
