@@ -33,12 +33,12 @@
 
 static void
 report_error(DBusError *err, const char *fmt, ...) {
-    static char buf[1024];
+    static char buf[4096];
     va_list args;
     va_start(args, fmt);
     int n = vsnprintf(buf, sizeof(buf), fmt, args);
     va_end(args);
-    snprintf(buf + n, sizeof(buf), ". DBUS error: %s", err->message);
+    if (n >= 0 && (size_t)n < (sizeof(buf) - 256)) snprintf(buf + n, sizeof(buf) - n, ". DBUS error: %s", err->message ? err->message : "(null)");
     _glfwInputError(GLFW_PLATFORM_ERROR, "%s", buf);
     dbus_error_free(err);
 }
@@ -214,33 +214,34 @@ typedef struct {
     void *user_data;
 } MethodResponse;
 
-static const char*
-format_message_error(DBusError *err) {
-    static char buf[1024];
-    snprintf(buf, sizeof(buf), "[%s] %s", err->name ? err->name : "", err->message);
-    return buf;
-}
-
 static void
 method_reply_received(DBusPendingCall *pending, void *user_data) {
     MethodResponse *res = (MethodResponse*)user_data;
-    DBusMessage *msg = dbus_pending_call_steal_reply(pending);
+    RAII_MSG(msg, dbus_pending_call_steal_reply(pending));
     if (msg) {
         DBusError err;
         dbus_error_init(&err);
-        if (dbus_set_error_from_message(&err, msg)) res->callback(NULL, format_message_error(&err), res->user_data);
+        if (dbus_set_error_from_message(&err, msg)) res->callback(NULL, &err, res->user_data);
         else res->callback(msg, NULL, res->user_data);
-        dbus_message_unref(msg);
     }
 }
 
 bool
-call_method_with_msg(DBusConnection *conn, DBusMessage *msg, int timeout, dbus_pending_callback callback, void *user_data) {
+call_method_with_msg(DBusConnection *conn, DBusMessage *msg, int timeout, dbus_pending_callback callback, void *user_data, bool block) {
     bool retval = false;
 #define REPORT(errs) _glfwInputError(GLFW_PLATFORM_ERROR, "Failed to call DBUS method: node=%s path=%s interface=%s method=%s, with error: %s", dbus_message_get_destination(msg), dbus_message_get_path(msg), dbus_message_get_interface(msg), dbus_message_get_member(msg), errs)
     if (callback) {
         DBusPendingCall *pending = NULL;
-        if (dbus_connection_send_with_reply(conn, msg, &pending, timeout)) {
+        if (block) {
+            DBusError error; dbus_error_init(&error);
+            RAII_MSG(reply, dbus_connection_send_with_reply_and_block(session_bus, msg, timeout, &error));
+            if (dbus_error_is_set(&error)) {
+                callback(reply, &error, user_data);
+                return false;
+            } else if (reply) {
+                callback(reply, NULL, user_data);
+            } else return false;
+        } else if (dbus_connection_send_with_reply(conn, msg, &pending, timeout)) {
             MethodResponse *res = malloc(sizeof(MethodResponse));
             if (!res) return false;
             res->callback = callback;
@@ -262,19 +263,18 @@ call_method_with_msg(DBusConnection *conn, DBusMessage *msg, int timeout, dbus_p
 }
 
 static bool
-call_method(DBusConnection *conn, const char *node, const char *path, const char *interface, const char *method, int timeout, dbus_pending_callback callback, void *user_data, va_list ap) {
+call_method(DBusConnection *conn, const char *node, const char *path, const char *interface, const char *method, int timeout, dbus_pending_callback callback, void *user_data, bool blocking, va_list ap) {
     if (!conn || !path) return false;
-    DBusMessage *msg = dbus_message_new_method_call(node, path, interface, method);
+    RAII_MSG(msg, dbus_message_new_method_call(node, path, interface, method));
     if (!msg) return false;
     bool retval = false;
 
     int firstarg = va_arg(ap, int);
     if ((firstarg == DBUS_TYPE_INVALID) || dbus_message_append_args_valist(msg, firstarg, ap)) {
-        retval = call_method_with_msg(conn, msg, timeout, callback, user_data);
+        retval = call_method_with_msg(conn, msg, timeout, callback, user_data, blocking);
     } else {
         _glfwInputError(GLFW_PLATFORM_ERROR, "Failed to call DBUS method: %s on node: %s and interface: %s could not add arguments", method, node, interface);
     }
-    dbus_message_unref(msg);
 
     return retval;
 }
@@ -284,7 +284,17 @@ glfw_dbus_call_method_with_reply(DBusConnection *conn, const char *node, const c
     bool retval;
     va_list ap;
     va_start(ap, user_data);
-    retval = call_method(conn, node, path, interface, method, timeout, callback, user_data, ap);
+    retval = call_method(conn, node, path, interface, method, timeout, callback, user_data, false, ap);
+    va_end(ap);
+    return retval;
+}
+
+bool
+glfw_dbus_call_blocking_method(DBusConnection *conn, const char *node, const char *path, const char *interface, const char *method, int timeout, dbus_pending_callback callback, void* user_data, ...) {
+    bool retval;
+    va_list ap;
+    va_start(ap, user_data);
+    retval = call_method(conn, node, path, interface, method, timeout, callback, user_data, true, ap);
     va_end(ap);
     return retval;
 }
@@ -294,7 +304,7 @@ glfw_dbus_call_method_no_reply(DBusConnection *conn, const char *node, const cha
     bool retval;
     va_list ap;
     va_start(ap, method);
-    retval = call_method(conn, node, path, interface, method, DBUS_TIMEOUT_USE_DEFAULT, NULL, NULL, ap);
+    retval = call_method(conn, node, path, interface, method, DBUS_TIMEOUT_USE_DEFAULT, NULL, NULL, false, ap);
     va_end(ap);
     return retval;
 }
