@@ -9,6 +9,7 @@ import (
 	"slices"
 	"strings"
 	"sync"
+	"sync/atomic"
 	"unicode/utf8"
 
 	"github.com/kovidgoyal/go-parallel"
@@ -27,6 +28,8 @@ type Preview interface {
 	Render(h *Handler, x, y, width, height int)
 	IsValidForColorScheme(light bool) bool
 	Unload()
+	IsReady() bool
+	String() string
 }
 
 type PreviewManager struct {
@@ -80,7 +83,9 @@ type MessagePreview struct {
 	trailers []string
 }
 
+func (p MessagePreview) String() string                  { return fmt.Sprintf("MessagePreview{%#v}", p.title) }
 func (p MessagePreview) IsValidForColorScheme(bool) bool { return true }
+func (p MessagePreview) IsReady() bool                   { return true }
 
 func (p MessagePreview) Unload() {}
 func (p MessagePreview) Render(h *Handler, x, y, width, height int) {
@@ -120,6 +125,7 @@ func write_file_metadata(abspath string, metadata fs.FileInfo, entries []fs.DirE
 	add := func(key, val string) { fmt.Fprintf(&buf, "%s: %s\n", key, val) }
 	ftype := metadata.Mode().Type()
 	const file_icon = " "
+	fmt.Fprintln(&buf, filepath.Base(abspath))
 	switch ftype {
 	case 0:
 		add("Size", humanize.Bytes(uint64(metadata.Size())))
@@ -171,12 +177,30 @@ func NewDirectoryPreview(abspath string, metadata fs.FileInfo) Preview {
 	return &MessagePreview{title: title, msg: header, trailers: extra}
 }
 
-func NewFileMetadataPreview(abspath string, metadata fs.FileInfo) Preview {
-	title := icons.IconForFileWithMode(filepath.Base(abspath), metadata.Mode().Type(), false) + "  File"
+func NewFileMetadataPreview(abspath string, metadata fs.FileInfo) *MessagePreview {
+	ext := filepath.Ext(abspath)
+	if ext == "" {
+		ext = "File"
+	}
+	title := icons.IconForFileWithMode(filepath.Base(abspath), metadata.Mode().Type(), false) + "  " + ext
 	h, t := write_file_metadata(abspath, metadata, nil)
 	return &MessagePreview{title: title, msg: h, trailers: t}
 }
 
+func NewFileMetadataPreviewWithError(abspath string, metadata fs.FileInfo, err error) *MessagePreview {
+	ext := filepath.Ext(abspath)
+	if ext == "" {
+		ext = "File"
+	}
+	title := icons.IconForFileWithMode(filepath.Base(abspath), metadata.Mode().Type(), false) + "  " + ext
+	h, t := write_file_metadata(abspath, metadata, nil)
+	ans := &MessagePreview{title: title, msg: h, trailers: t}
+	lines := style.WrapTextAsLines(err.Error(), 30, style.WrapOptions{})
+	ans.trailers = append(ans.trailers, "")
+	ans.trailers = append(ans.trailers, lines...)
+	return ans
+}
+
 type highlighed_data struct {
 	text  string
 	light bool
@@ -186,14 +210,21 @@ type highlighed_data struct {
 type TextFilePreview struct {
 	plain_text, highlighted_text string
 	highlighted_chan             chan highlighed_data
+	ready                        atomic.Bool
 	light                        bool
 	path                         string
+	metadata                     fs.FileInfo
 }
 
-func (p TextFilePreview) IsValidForColorScheme(light bool) bool { return p.light == light }
+func (p *TextFilePreview) String() string {
+	return fmt.Sprintf("TextFilePreview{%#v, ready: %v}", p.path, p.ready.Load())
+}
+
+func (p *TextFilePreview) IsValidForColorScheme(light bool) bool { return p.light == light }
 
 func (p *TextFilePreview) Unload() {}
 
+func (p *TextFilePreview) IsReady() bool { return p.ready.Load() || p.highlighted_chan == nil }
 func (p *TextFilePreview) Render(h *Handler, x, y, width, height int) {
 	if p.highlighted_chan != nil {
 		select {
@@ -212,7 +243,8 @@ func (p *TextFilePreview) Render(h *Handler, x, y, width, height int) {
 	s := utils.NewLineScanner(text)
 	buf := strings.Builder{}
 	buf.Grow(1024 * height)
-	for num := 0; s.Scan() && num < height; num++ {
+	title := icons.IconForPath(p.path) + "  " + filepath.Base(p.path) + fmt.Sprintf(" %s", humanize.Bytes(uint64(p.metadata.Size())))
+	for num := 1 + h.render_wrapped_text_in_region(title, x, y, width, height, true); s.Scan() && num < height; num++ {
 		line := s.Text()
 		truncated := wcswidth.TruncateToVisualLength(line, width)
 		buf.WriteString(fmt.Sprintf(loop.MoveCursorToTemplate, y+num, x))
@@ -234,7 +266,8 @@ func NewTextFilePreview(abspath string, metadata fs.FileInfo, highlighted_chan c
 	if !utf8.ValidString(text) {
 		text = "Error: not valid utf-8 text"
 	}
-	return &TextFilePreview{path: abspath, plain_text: sanitize(text), highlighted_chan: highlighted_chan, light: use_light_colors}
+	return &TextFilePreview{
+		path: abspath, plain_text: sanitize(text), highlighted_chan: highlighted_chan, light: use_light_colors, metadata: metadata}
 }
 
 type style_resolver struct {
@@ -256,22 +289,20 @@ func (s style_resolver) TextForPath(path string) (string, error) {
 	return "", err
 }
 
-func (pm *PreviewManager) highlight_file_async(path string, output chan highlighed_data) {
+func (pm *PreviewManager) highlight_file_async(path string, output chan highlighed_data, ready *atomic.Bool) {
 	s := style_resolver{light: use_light_colors, syntax_aliases: pm.settings.SyntaxAliases()}
 	s.light_style, s.dark_style = pm.settings.HighlightStyles()
 	go func() {
 		defer func() {
 			if r := recover(); r != nil {
 				err := parallel.Format_stacktrace_on_panic(r, 1)
-				debugprintln(fmt.Sprintf("Failed to highlight: %s with panic: %s", path, err))
+				output <- highlighed_data{err: err, light: s.light}
 			}
 			close(output)
+			ready.Store(true)
 			pm.WakeupMainThread()
 		}()
 		highlighted, err := pm.highlighter.HighlightFile(path, &s)
-		if err != nil {
-			debugprintln(fmt.Sprintf("Failed to highlight: %s with error: %s", path, err))
-		}
 		output <- highlighed_data{text: highlighted, err: err, light: s.light}
 	}()
 }
@@ -301,24 +332,54 @@ func (pm *PreviewManager) preview_for(abspath string, ftype fs.FileMode) (ans Pr
 		}
 		return NewDirectoryPreview(abspath, s)
 	}
-	mt := utils.GuessMimeType(filepath.Base(abspath))
+	fname := filepath.Base(abspath)
+	mt := utils.GuessMimeType(fname)
+	for _, q := range previewers {
+		if q.matches(fname, mt) {
+			return NewCmdPreview(abspath, s, pm.settings, pm.WakeupMainThread, q)
+		}
+	}
 	const MAX_TEXT_FILE_SIZE = 16 * 1024 * 1024
 	if s.Size() <= MAX_TEXT_FILE_SIZE && (utils.KnownTextualMimes[mt] || strings.HasPrefix(mt, "text/")) {
 		ch := make(chan highlighed_data, 2)
-		pm.highlight_file_async(abspath, ch)
-		return NewTextFilePreview(abspath, s, ch, pm.highlighter.Sanitize)
+		ans := NewTextFilePreview(abspath, s, ch, pm.highlighter.Sanitize)
+		if p, ok := ans.(*TextFilePreview); ok {
+			pm.highlight_file_async(abspath, ch, &p.ready)
+		}
+		return ans
 	}
-	if strings.HasPrefix(mt, "image/") {
+	switch {
+	case strings.HasPrefix(mt, "image/"):
 		var r ImagePreviewRenderer
 		if ans, err := NewImagePreview(abspath, s, pm.settings, pm.WakeupMainThread, r); err == nil {
 			return ans
 		} else {
 			return NewErrorPreview(err)
 		}
+
+	case strings.HasPrefix(mt, "video/"):
+		return NewFFMpegPreview(abspath, s, pm.settings, pm.WakeupMainThread)
+
+	case IsSupportedArchiveFile(abspath):
+		return NewArchivePeview(abspath, s, pm.settings, pm.WakeupMainThread)
+
+	case IsSupportedByCalibre(abspath):
+		return NewCalibrePreview(abspath, s, pm.settings, pm.WakeupMainThread)
 	}
 	return NewFileMetadataPreview(abspath, s)
 }
 
+func (h *Handler) clear_cached_previews() {
+	if h.last_rendered_preview != nil {
+		h.last_rendered_preview.Unload()
+		h.last_rendered_preview = nil
+	}
+	if h.prev_preview_for_smooth_transition != nil {
+		h.prev_preview_for_smooth_transition.Unload()
+		h.prev_preview_for_smooth_transition = nil
+	}
+}
+
 func (h *Handler) draw_preview_content(x, y, width, height int) {
 	matches, _ := h.get_results()
 	r := matches.At(h.state.CurrentIndex())
@@ -326,15 +387,36 @@ func (h *Handler) draw_preview_content(x, y, width, height int) {
 		h.render_wrapped_text_in_region("No preview available", x, y, width, height, false)
 		return
 	}
-	abspath := filepath.Join(h.state.CurrentDir(), r.text)
+	render := func() {
+		p := h.last_rendered_preview
+		if p.IsReady() || h.prev_preview_for_smooth_transition == nil {
+			p.Render(h, x, y, width, height)
+			if h.prev_preview_for_smooth_transition != nil {
+				h.prev_preview_for_smooth_transition.Unload()
+				h.prev_preview_for_smooth_transition = nil
+			}
+		} else {
+			h.prev_preview_for_smooth_transition.Render(h, x, y, width, height)
+		}
+	}
+
+	abspath := h.current_abspath()
 	if h.last_rendered_preview != nil {
-		h.last_rendered_preview.Unload()
+		if abspath == h.last_rendered_preview_abspath {
+			render()
+			return
+		}
+		if h.prev_preview_for_smooth_transition != nil {
+			h.prev_preview_for_smooth_transition.Unload()
+		}
+		h.prev_preview_for_smooth_transition = h.last_rendered_preview
 		h.last_rendered_preview = nil
 	}
 	if p := h.preview_manager.preview_for(abspath, r.ftype); p == nil {
 		h.render_wrapped_text_in_region("No preview available", x, y, width, height, false)
 	} else {
 		h.last_rendered_preview = p
-		p.Render(h, x, y, width, height)
+		h.last_rendered_preview_abspath = abspath
+		render()
 	}
 }
