@@ -6,6 +6,7 @@ import (
 	"path/filepath"
 	"strings"
 
+	"github.com/kovidgoyal/go-parallel"
 	"github.com/kovidgoyal/kitty/tools/cli"
 	"github.com/kovidgoyal/kitty/tools/tui/loop"
 	"github.com/kovidgoyal/kitty/tools/utils"
@@ -26,8 +27,7 @@ func FilePromptCompleter(getcwd func() string) func(string, string) *cli.Complet
 	return func(before_cursor, after_cursor string) (ans *cli.Completions) {
 		defer func() {
 			if r := recover(); r != nil {
-				text, err := utils.Format_stacktrace_on_panic(r)
-				debugprintln(text)
+				err := parallel.Format_stacktrace_on_panic(r, 1)
 				debugprintln(err)
 			}
 		}()
