@@ -6,8 +6,8 @@ import (
 	"encoding/binary"
 	"encoding/json"
 	"fmt"
+	"github.com/kovidgoyal/go-shm"
 	"github.com/kovidgoyal/kitty"
-	"github.com/kovidgoyal/kitty/tools/utils/shm"
 	"io/fs"
 	"os"
 	"os/exec"
