@@ -1,14 +1,12 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
-from typing import Callable, Tuple
+from collections.abc import Callable
 
 from kitty.fast_data_types import truncate_point_for_length, wcswidth
 from kitty.key_encoding import EventType, KeyEvent
 
-from .operations import (
-    RESTORE_CURSOR, SAVE_CURSOR, move_cursor_by, set_cursor_shape
-)
+from .operations import RESTORE_CURSOR, SAVE_CURSOR, move_cursor_by, set_cursor_shape
 
 
 class LineEdit:
@@ -22,7 +20,7 @@ class LineEdit:
         self.cursor_pos = 0
         self.pending_bell = False
 
-    def split_at_cursor(self, delta: int = 0) -> Tuple[str, str]:
+    def split_at_cursor(self, delta: int = 0) -> tuple[str, str]:
         pos = max(0, self.cursor_pos + delta)
         x = truncate_point_for_length(self.current_input, pos) if pos else 0
         before, after = self.current_input[:x], self.current_input[x:]
@@ -53,7 +51,7 @@ class LineEdit:
             write('\r')
             if cursor_pos:
                 write(move_cursor_by(cursor_pos, 'right'))
-            write(set_cursor_shape('bar'))
+            write(set_cursor_shape('beam'))
 
     def add_text(self, text: str) -> None:
         if self.current_input:
