@@ -3,26 +3,28 @@
 package unicode_input
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
+	"math"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
 	"unicode"
+	"unicode/utf8"
 
-	"kitty/tools/cli"
-	"kitty/tools/tty"
-	"kitty/tools/tui"
-	"kitty/tools/tui/loop"
-	"kitty/tools/tui/readline"
-	"kitty/tools/unicode_names"
-	"kitty/tools/utils"
-	"kitty/tools/utils/style"
-	"kitty/tools/wcswidth"
-
-	"golang.org/x/exp/slices"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/tui"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/tui/readline"
+	"github.com/kovidgoyal/kitty/tools/unicode_names"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/style"
+	"github.com/kovidgoyal/kitty/tools/wcswidth"
 )
 
 var _ = fmt.Print
@@ -50,7 +52,7 @@ func build_sets() {
 	}
 	EMOTICONS_SET = make([]rune, 0, 0x1f64f-0x1f600+1)
 	for i := 0x1f600; i <= 0x1f64f; i++ {
-		DEFAULT_SET = append(DEFAULT_SET, rune(i))
+		EMOTICONS_SET = append(EMOTICONS_SET, rune(i))
 	}
 }
 
@@ -70,8 +72,8 @@ func parse_favorites(raw string) (ans []rune) {
 			line = line[:idx]
 		}
 		code_text, _, _ := strings.Cut(line, " ")
-		code, err := strconv.ParseUint(code_text, 16, 32)
-		if err == nil && codepoint_ok(rune(code)) {
+		code, err := strconv.ParseInt(code_text, 16, 32)
+		if err == nil && code <= utf8.MaxRune && codepoint_ok(rune(code)) {
 			ans = append(ans, rune(code))
 		}
 	}
@@ -208,7 +210,6 @@ func is_index(word string) bool {
 }
 
 func (self *handler) update_codepoints() {
-	var index_word uint64
 	var q checkpoints_key
 	q.mode = self.mode
 	q.index_word = -1
@@ -232,8 +233,9 @@ func (self *handler) update_codepoints() {
 					if i > 0 && is_index(w) {
 						iw := words[i]
 						words = words[:i]
-						index_word, _ = strconv.ParseUint(strings.TrimLeft(iw, INDEX_CHAR), INDEX_BASE, 32)
-						q.index_word = int(index_word)
+						if index_word, perr := strconv.ParseInt(strings.TrimLeft(iw, INDEX_CHAR), INDEX_BASE, 0); perr == nil {
+							q.index_word = int(index_word)
+						}
 						break
 					}
 				}
@@ -388,17 +390,18 @@ func (self *handler) switch_mode(mode Mode) {
 
 func (self *handler) handle_hex_key_event(event *loop.KeyEvent) {
 	text := self.rl.AllText()
-	val, err := strconv.ParseUint(text, 16, 32)
+	uval, err := strconv.ParseUint(text, 16, 32)
 	new_val := -1
-	if err != nil {
+	if err != nil || uval > math.MaxInt {
 		return
 	}
+	val := int(uval)
 	if event.MatchesPressOrRepeat("tab") {
-		new_val = int(val) + 10
+		new_val = val + 10
 	} else if event.MatchesPressOrRepeat("up") {
-		new_val = int(val) + 1
+		new_val = val + 1
 	} else if event.MatchesPressOrRepeat("down") {
-		new_val = utils.Max(32, int(val)-1)
+		new_val = max(32, val-1)
 	}
 	if new_val > -1 {
 		event.Handled = true
@@ -443,7 +446,7 @@ func (self *handler) handle_favorites_key_event(event *loop.KeyEvent) {
 				self.lp.Quit(1)
 				return
 			}
-			err = utils.AtomicUpdateFile(fp, utils.UnsafeStringToBytes(raw), 0o600)
+			err = utils.AtomicUpdateFile(fp, bytes.NewReader(utils.UnsafeStringToBytes(raw)), 0o600)
 			if err != nil {
 				self.err = fmt.Errorf("Failed to write to favorites file %s with error: %w", fp, err)
 				self.lp.Quit(1)
