@@ -9,7 +9,6 @@
 
 #include "disk-cache.h"
 #include "safe-wrappers.h"
-#include "kitty-uthash.h"
 #include "simd-string.h"
 #include "loop-utils.h"
 #include "fast-file-copy.h"
@@ -21,25 +20,56 @@
 #include <fcntl.h>
 #include <time.h>
 
+typedef struct CacheKey {
+    void *hash_key;
+    unsigned short hash_keylen;
+} CacheKey;
 
 typedef struct {
-    void *hash_key;
     uint8_t *data;
     size_t data_sz;
-    unsigned short hash_keylen;
-    bool written_to_disk;
+    bool written_to_disk, uses_encryption;
     off_t pos_in_cache_file;
     uint8_t encryption_key[64];
-    UT_hash_handle hh;
-} CacheEntry;
+} CacheValue;
+
+#define NAME cache_map
+#define KEY_TY CacheKey
+#define VAL_TY CacheValue*
+static uint64_t key_hash(KEY_TY k);
+#define HASH_FN key_hash
+static bool keys_are_equal(CacheKey a, CacheKey b) { return a.hash_keylen == b.hash_keylen && memcmp(a.hash_key, b.hash_key, a.hash_keylen) == 0; }
+#define CMPR_FN keys_are_equal
+static void free_cache_value(CacheValue *cv) { free(cv->data); cv->data = NULL; free(cv); }
+static void free_cache_key(CacheKey cv) { free(cv.hash_key); cv.hash_key = NULL; }
+#define KEY_DTOR_FN free_cache_key
+#define VAL_DTOR_FN free_cache_value
+#include "kitty-verstable.h"
+static uint64_t key_hash(CacheKey k) { return vt_hash_bytes(k.hash_key, k.hash_keylen); }
+#define cache_map_for_loop(i) vt_create_for_loop(cache_map_itr, i, &self->map)
 
 typedef struct Hole {
     off_t pos, size;
 } Hole;
 
+#define NAME hole_pos_map
+#define KEY_TY off_t
+#define VAL_TY off_t
+#include "kitty-verstable.h"
+#define hole_pos_map_for_loop(i) vt_create_for_loop(hole_pos_map_itr, i, &self->holes.pos_map)
+
+typedef struct PosList { size_t count, capacity; off_t *positions; } PosList;
+#define NAME hole_size_map
+#define KEY_TY off_t
+#define VAL_TY PosList
+static void free_pos_list(PosList p) { free(p.positions); }
+#define VAL_DTOR_FN free_pos_list
+#include "kitty-verstable.h"
+#define hole_size_map_for_loop(i) vt_create_for_loop(hole_size_map_itr, i, &holes->size_map)
+
 typedef struct Holes {
-    Hole *items;
-    size_t capacity, count;
+    hole_pos_map pos_map, end_pos_map;
+    hole_size_map size_map;
     off_t largest_hole_size;
 } Holes;
 
@@ -48,22 +78,19 @@ typedef struct {
     char *cache_dir;
     int cache_file_fd;
     Py_ssize_t small_hole_threshold;
+    unsigned int defrag_factor;
     pthread_mutex_t lock;
     pthread_t write_thread;
     bool thread_started, lock_inited, loop_data_inited, shutting_down, fully_initialized;
     LoopData loop_data;
-    CacheEntry *entries, currently_writing;
+    struct { CacheValue val; CacheKey key; } currently_writing;
+    cache_map map;
     Holes holes;
     unsigned long long total_size;
+    off_t end_of_data_offset;
+    bool needs_encryption;
 } DiskCache;
 
-void
-free_cache_entry(CacheEntry *e) {
-    if (e->hash_key) { free(e->hash_key); e->hash_key = NULL; }
-    if (e->data) { free(e->data); e->data = NULL; }
-    free(e);
-}
-
 #define mutex(op) pthread_mutex_##op(&self->lock)
 
 static PyObject*
@@ -73,6 +100,8 @@ new_diskcache_object(PyTypeObject *type, PyObject UNUSED *args, PyObject UNUSED
     if (self) {
         self->cache_file_fd = -1;
         self->small_hole_threshold = 512;
+        self->defrag_factor = 2;
+        self->needs_encryption = true;
     }
     return (PyObject*) self;
 }
@@ -94,14 +123,16 @@ open_cache_file_without_tmpfile(const char *cache_path) {
 }
 
 static int
-open_cache_file(const char *cache_path) {
+open_cache_file(const char *cache_path, bool *opened_securely) {
     int fd = -1;
+    *opened_securely = false;
 #ifdef O_TMPFILE
     while (fd < 0) {
         fd = safe_open(cache_path, O_TMPFILE | O_CLOEXEC | O_EXCL | O_RDWR, S_IRUSR | S_IWUSR);
         if (fd > -1 || errno != EINTR) break;
     }
     if (fd == -1) fd = open_cache_file_without_tmpfile(cache_path);
+    else *opened_securely = true;
 #else
     fd = open_cache_file_without_tmpfile(cache_path);
 #endif
@@ -110,27 +141,27 @@ open_cache_file(const char *cache_path) {
 
 // Write loop {{{
 
-static off_t
-size_of_cache_file(DiskCache *self) {
-    return lseek(self->cache_file_fd, 0, SEEK_END);
-}
-
-size_t
-disk_cache_size_on_disk(PyObject *self) {
-    if (((DiskCache*)self)->cache_file_fd > -1) {
-        off_t ans = size_of_cache_file((DiskCache*)self);
-        return MAX(0, ans);
-    }
-    return 0;
-}
-
 typedef struct {
-    uint8_t hash_key[MAX_KEY_SIZE];
-    unsigned short hash_keylen;
+    CacheKey key;
     off_t old_offset, new_offset;
     size_t data_sz;
 } DefragEntry;
 
+static CacheKey
+keydup(CacheKey k) {
+    CacheKey ans = {.hash_key=malloc(k.hash_keylen), .hash_keylen=k.hash_keylen};
+    if (ans.hash_key) memcpy(ans.hash_key, k.hash_key, k.hash_keylen);
+    return ans;
+}
+
+static void
+cleanup_holes(Holes *holes) {
+    vt_cleanup(&holes->size_map);
+    vt_cleanup(&holes->pos_map);
+    vt_cleanup(&holes->end_pos_map);
+    holes->largest_hole_size = 0;
+}
+
 static void
 defrag(DiskCache *self) {
     int new_cache_file = -1;
@@ -138,11 +169,12 @@ defrag(DiskCache *self) {
     RAII_FreeFastFileCopyBuffer(fcb);
     bool lock_released = false, ok = false;
 
-    off_t size_on_disk = size_of_cache_file(self);
+    off_t size_on_disk = self->end_of_data_offset;
     if (size_on_disk <= 0) goto cleanup;
-    size_t num_entries = HASH_COUNT(self->entries);
+    size_t num_entries = vt_size(&self->map);
     if (!num_entries) goto cleanup;
-    new_cache_file = open_cache_file(self->cache_dir);
+    bool opened_securely;
+    new_cache_file = open_cache_file(self->cache_dir, &opened_securely);
     if (new_cache_file < 0) {
         perror("Failed to open second file for defrag of disk cache");
         goto cleanup;
@@ -150,15 +182,15 @@ defrag(DiskCache *self) {
     defrag_entries = calloc(num_entries, sizeof(DefragEntry));
     if (!defrag_entries) goto cleanup;
     size_t total_data_size = 0, num_entries_to_defrag = 0;
-    CacheEntry *tmp, *s;
-    HASH_ITER(hh, self->entries, s, tmp) {
+    cache_map_for_loop(i) {
+        CacheValue *s = i.data->val;
         if (s->pos_in_cache_file > -1 && s->data_sz) {
             total_data_size += s->data_sz;
             DefragEntry *e = defrag_entries + num_entries_to_defrag++;
-            e->hash_keylen = s->hash_keylen;
             e->old_offset = s->pos_in_cache_file;
             e->data_sz = s->data_sz;
-            if (s->hash_key) memcpy(e->hash_key, s->hash_key, s->hash_keylen);
+            e->key = keydup(i.data->key);  // have to dup the key as we release the mutex and another thread might free the underlying key.
+            if (!e->key.hash_key) { fprintf(stderr, "Failed to allocate space for keydup in defrag\n"); goto cleanup; }
         }
     }
     if (ftruncate(new_cache_file, total_data_size) != 0) {
@@ -179,83 +211,140 @@ defrag(DiskCache *self) {
         e->new_offset = current_pos;
         current_pos += e->data_sz;
     }
-    self->holes.count = 0;
-    self->holes.largest_hole_size = 0;
     ok = true;
 
 cleanup:
     if (lock_released) mutex(lock);
     if (ok) {
+        cleanup_holes(&self->holes);
         safe_close(self->cache_file_fd, __FILE__, __LINE__);
         self->cache_file_fd = new_cache_file; new_cache_file = -1;
         for (size_t i = 0; i < num_entries_to_defrag; i++) {
             DefragEntry *e = defrag_entries + i;
-            s = NULL;
-            HASH_FIND(hh, self->entries, e->hash_key, e->hash_keylen, s);
-            if (s) s->pos_in_cache_file = e->new_offset;
+            cache_map_itr i = vt_get(&self->map, e->key);
+            if (!vt_is_end(i)) i.data->val->pos_in_cache_file = e->new_offset;
+            free(e->key.hash_key);
         }
+        self->end_of_data_offset = lseek(self->cache_file_fd, 0, SEEK_CUR);
+        self->needs_encryption = !opened_securely;
     }
     if (new_cache_file > -1) safe_close(new_cache_file, __FILE__, __LINE__);
 }
 
+static void
+append_position(PosList *p, off_t pos) {
+    ensure_space_for(p, positions, off_t, p->count + 1, capacity, 8, false);
+    p->positions[p->count++] = pos;
+}
+
+static void
+add_hole_to_maps(Holes *holes, Hole h) {
+    if (vt_is_end(vt_insert(&holes->pos_map, h.pos, h.size))) fatal("Out of memory");
+    if (vt_is_end(vt_insert(&holes->end_pos_map, h.pos + h.size, h.size))) fatal("Out of memory");
+    hole_size_map_itr i = vt_get_or_insert(&holes->size_map, h.size, (PosList){0});
+    if (vt_is_end(i)) fatal("Out of memory");
+    append_position(&(i.data->val), h.pos);
+    holes->largest_hole_size = MAX(h.size, holes->largest_hole_size);
+}
+
+static void
+update_largest_hole_size(Holes *holes) {
+    holes->largest_hole_size = 0;
+    hole_size_map_for_loop(i) {
+        if (i.data->key > holes->largest_hole_size) holes->largest_hole_size = i.data->key;
+    }
+}
+
+static void
+remove_hole_from_maps_itr(Holes *holes, Hole h, hole_size_map_itr i, size_t pos_in_sizes_array) {
+    vt_erase(&holes->pos_map, h.pos); vt_erase(&holes->end_pos_map, h.pos + h.size);
+    if (i.data->val.count <= 1) {
+        vt_erase_itr(&holes->size_map, i);
+        if (h.size > holes->largest_hole_size) update_largest_hole_size(holes);
+    } else remove_i_from_array(i.data->val.positions, pos_in_sizes_array, i.data->val.count);
+}
+
+static void
+remove_hole_from_maps(Holes *holes, Hole h) {
+    hole_size_map_itr i = vt_get(&holes->size_map, h.size);
+    for (size_t x = 0; x < i.data->val.count; x++) {
+        if (i.data->val.positions[x] == h.pos) {
+            remove_hole_from_maps_itr(holes, h, vt_get(&holes->size_map, h.size), x);
+            return;
+        }
+    }
+}
+
 static bool
 find_hole_to_use(DiskCache *self, const off_t required_sz) {
     if (self->holes.largest_hole_size < required_sz) return false;
-    off_t largest_hole_size = 0;
-    bool found = false;
-    ssize_t remove_at = -1;
-    for (size_t i = 0; i < self->holes.count; i++) {
-        Hole *h = self->holes.items + i;
-        if (!found && h->size >= required_sz) {
-            found = true;
-            self->currently_writing.pos_in_cache_file = h->pos;
-            bool was_largest_hole = h->size == self->holes.largest_hole_size;
-            h->size -= required_sz;
-            h->pos += required_sz;
-            if (h->size <= self->small_hole_threshold) remove_at = i;
-            if (!was_largest_hole) {
-                if (remove_at < 0) return found;
-                largest_hole_size = self->holes.largest_hole_size;
-                break;
-            }
+    hole_size_map_itr i = vt_get(&self->holes.size_map, required_sz);
+    if (vt_is_end(i)) {
+        for (i = vt_first(&self->holes.size_map); !vt_is_end(i); i = vt_next(i)) {
+            if (i.data->key >= required_sz) break;
         }
-        if (h->size > largest_hole_size) largest_hole_size = h->size;
     }
-    self->holes.largest_hole_size = largest_hole_size;
-    if (remove_at > -1) remove_i_from_array(self->holes.items, (size_t)remove_at, self->holes.count);
-    return found;
+    if (vt_is_end(i)) return false;
+    Hole h = {.pos=i.data->val.positions[i.data->val.count-1], .size=i.data->key};
+    remove_hole_from_maps_itr(&self->holes, h, i, i.data->val.count-1);
+    self->currently_writing.val.pos_in_cache_file = h.pos;
+    if (required_sz < h.size) {
+        h.pos += required_sz; h.size -= required_sz;
+        if (h.size > self->small_hole_threshold) add_hole_to_maps(&self->holes, h);
+    }
+    return true;
 }
 
 static inline bool
 needs_defrag(DiskCache *self) {
-    off_t size_on_disk = size_of_cache_file(self);
-    return self->total_size && size_on_disk > 0 && (size_t)size_on_disk > self->total_size * 2;
+    return self->total_size && self->end_of_data_offset > 0 && (size_t)self->end_of_data_offset > self->total_size * self->defrag_factor;
 }
 
 static void
-add_hole(DiskCache *self, off_t pos, off_t size) {
+add_hole(DiskCache *self, const off_t pos, const off_t size) {
     if (size <= self->small_hole_threshold) return;
-    Hole *h;
-    if (self->holes.count) {
-        // see if we can find a hole that ends where we start and merge
-        // look through the prev at most 128 holes for a match
-        h = &self->holes.items[self->holes.count-1];
-        for (size_t i = 0; i < MIN(self->holes.count, 128u); i++, h--) {
-            if (h->pos + h->size == pos) {
-                h->size += size;
-                self->holes.largest_hole_size = MAX(self->holes.largest_hole_size, h->size);
-                return;
+    if (vt_size(&self->holes.pos_map)) {
+        // See if we can find a neighboring hole to merge this hole into
+        // First look for a hole after us
+        off_t end_pos = pos + size;
+        hole_pos_map_itr i = vt_get(&self->holes.pos_map, end_pos);
+        Hole original_hole, new_hole;
+        bool found = false;
+        if (vt_is_end(i)) {
+            // Now look for a hole before us
+            i = vt_get(&self->holes.end_pos_map, pos);
+            if (!vt_is_end(i)) {
+                original_hole.pos = i.data->key - i.data->val; original_hole.size = i.data->val;
+                new_hole.pos = original_hole.pos; new_hole.size = original_hole.size + size;
+                found = true;
+            }
+        } else {
+            original_hole.pos = i.data->key; original_hole.size = i.data->val;
+            new_hole.pos = pos; new_hole.size = original_hole.size + size;
+            found = true;
+            // there could be a hole before us as well
+            i = vt_get(&self->holes.end_pos_map, pos);
+            if (!vt_is_end(i)) {
+                self->holes.largest_hole_size = MAX(self->holes.largest_hole_size, new_hole.size);
+                remove_hole_from_maps(&self->holes, original_hole);
+                original_hole.pos = i.data->key - i.data->val; original_hole.size = i.data->val;
+                new_hole.pos = original_hole.pos; new_hole.size += original_hole.size;
             }
         }
+        if (found) {
+            // prevent remove_hole_from_maps updating largest hole size
+            self->holes.largest_hole_size = MAX(self->holes.largest_hole_size, new_hole.size);
+            remove_hole_from_maps(&self->holes, original_hole);
+            add_hole_to_maps(&self->holes, new_hole);
+            return;
+        }
     }
-    ensure_space_for(&self->holes, items, Hole, 1, capacity, 64, false);
-    h = &self->holes.items[self->holes.count++];
-    h->pos = pos; h->size = size;
-    self->holes.largest_hole_size = MAX(self->holes.largest_hole_size, h->size);
+    Hole h = {.pos=pos, .size=size };
+    add_hole_to_maps(&self->holes, h);
 }
 
 static void
-remove_from_disk(DiskCache *self, CacheEntry *s) {
+remove_from_disk(DiskCache *self, CacheValue *s) {
     if (s->written_to_disk) {
         s->written_to_disk = false;
         if (s->data_sz && s->pos_in_cache_file > -1) {
@@ -267,20 +356,24 @@ remove_from_disk(DiskCache *self, CacheEntry *s) {
 
 static bool
 find_cache_entry_to_write(DiskCache *self) {
-    CacheEntry *tmp, *s;
     if (needs_defrag(self)) defrag(self);
-    HASH_ITER(hh, self->entries, s, tmp) {
+    cache_map_for_loop(i) {
+        CacheValue *s = i.data->val;
         if (!s->written_to_disk) {
             if (s->data) {
-                if (self->currently_writing.data) free(self->currently_writing.data);
-                self->currently_writing.data = s->data;
+                if (self->currently_writing.val.data) free(self->currently_writing.val.data);
+                self->currently_writing.val.data = s->data;
                 s->data = NULL;
-                self->currently_writing.data_sz = s->data_sz;
-                self->currently_writing.pos_in_cache_file = -1;
-                xor_data64(s->encryption_key, self->currently_writing.data, s->data_sz);
-                self->currently_writing.hash_keylen = MIN(s->hash_keylen, MAX_KEY_SIZE);
-                memcpy(self->currently_writing.hash_key, s->hash_key, self->currently_writing.hash_keylen);
-                find_hole_to_use(self, self->currently_writing.data_sz);
+                self->currently_writing.val.data_sz = s->data_sz;
+                self->currently_writing.val.pos_in_cache_file = -1;
+                s->uses_encryption = false;
+                if (self->needs_encryption && secure_random_bytes(s->encryption_key, sizeof(s->encryption_key))) {
+                    xor_data64(s->encryption_key, self->currently_writing.val.data, s->data_sz);
+                    s->uses_encryption = true;
+                }
+                self->currently_writing.key.hash_keylen = MIN(i.data->key.hash_keylen, MAX_KEY_SIZE);
+                memcpy(self->currently_writing.key.hash_key, i.data->key.hash_key, self->currently_writing.key.hash_keylen);
+                find_hole_to_use(self, self->currently_writing.val.data_sz);
                 return true;
             }
             s->written_to_disk = true;
@@ -293,47 +386,59 @@ find_cache_entry_to_write(DiskCache *self) {
 
 static bool
 write_dirty_entry(DiskCache *self) {
-    size_t left = self->currently_writing.data_sz;
-    uint8_t *p = self->currently_writing.data;
-    if (self->currently_writing.pos_in_cache_file < 0) {
-        self->currently_writing.pos_in_cache_file = size_of_cache_file(self);
-        if (self->currently_writing.pos_in_cache_file < 0) {
+    size_t left = self->currently_writing.val.data_sz;
+    uint8_t *p = self->currently_writing.val.data;
+    if (self->currently_writing.val.pos_in_cache_file < 0) {
+        self->currently_writing.val.pos_in_cache_file = self->end_of_data_offset;
+        if (self->currently_writing.val.pos_in_cache_file < 0) {
             perror("Failed to seek in disk cache file");
             return false;
         }
     }
-    off_t offset = self->currently_writing.pos_in_cache_file;
+    off_t offset = self->currently_writing.val.pos_in_cache_file;
     while (left > 0) {
         ssize_t n = pwrite(self->cache_file_fd, p, left, offset);
         if (n < 0) {
             if (errno == EINTR || errno == EAGAIN) continue;
             perror("Failed to write to disk-cache file");
-            self->currently_writing.pos_in_cache_file = -1;
+            self->currently_writing.val.pos_in_cache_file = -1;
             return false;
         }
         if (n == 0) {
             fprintf(stderr, "Failed to write to disk-cache file with zero return\n");
-            self->currently_writing.pos_in_cache_file = -1;
+            self->currently_writing.val.pos_in_cache_file = -1;
             return false;
         }
         left -= n;
         p += n;
         offset += n;
+        self->end_of_data_offset = MAX(self->end_of_data_offset, offset);
     }
     return true;
 }
 
 static void
 retire_currently_writing(DiskCache *self) {
-    CacheEntry *s = NULL;
-    HASH_FIND(hh, self->entries, self->currently_writing.hash_key, self->currently_writing.hash_keylen, s);
-    if (s) {
-        s->written_to_disk = true;
-        s->pos_in_cache_file = self->currently_writing.pos_in_cache_file;
+    cache_map_itr i = vt_get(&self->map, self->currently_writing.key);
+    if (!vt_is_end(i)) {
+        i.data->val->written_to_disk = true;
+        i.data->val->pos_in_cache_file = self->currently_writing.val.pos_in_cache_file;
     }
-    free(self->currently_writing.data);
-    self->currently_writing.data = NULL;
-    self->currently_writing.data_sz = 0;
+    free(self->currently_writing.val.data);
+    self->currently_writing.val.data = NULL;
+    self->currently_writing.val.data_sz = 0;
+}
+
+static int
+clear_disk_cache_with_lock_held(DiskCache *self) {
+    vt_cleanup(&self->map);
+    cleanup_holes(&self->holes);
+    self->total_size = 0;
+    self->end_of_data_offset = 0;
+    if (self->cache_file_fd > -1) {
+        if (ftruncate(self->cache_file_fd, 0) == -1) return errno;
+    }
+    return 0;
 }
 
 static void*
@@ -348,7 +453,7 @@ write_loop(void *data) {
     while (!self->shutting_down) {
         mutex(lock);
         found_dirty_entry = find_cache_entry_to_write(self);
-        size_t count = HASH_COUNT(self->entries);
+        size_t count = vt_size(&self->map);
         mutex(unlock);
         if (found_dirty_entry) {
             write_dirty_entry(self);
@@ -358,9 +463,8 @@ write_loop(void *data) {
             continue;
         } else if (!count) {
             mutex(lock);
-            if (self->cache_file_fd > -1) {
-                if (ftruncate(self->cache_file_fd, 0) == 0) lseek(self->cache_file_fd, 0, SEEK_END);
-            }
+            count = vt_size(&self->map);
+            if (!count) clear_disk_cache_with_lock_held(self);  // failure to truncate is not fatal
             mutex(unlock);
         }
 
@@ -380,9 +484,9 @@ ensure_state(DiskCache *self) {
         if (!init_loop_data(&self->loop_data, 0)) { PyErr_SetFromErrno(PyExc_OSError); return false; }
         self->loop_data_inited = true;
     }
-    if (!self->currently_writing.hash_key) {
-        self->currently_writing.hash_key = malloc(MAX_KEY_SIZE);
-        if (!self->currently_writing.hash_key) { PyErr_NoMemory(); return false; }
+    if (!self->currently_writing.key.hash_key) {
+        self->currently_writing.key.hash_key = malloc(MAX_KEY_SIZE);
+        if (!self->currently_writing.key.hash_key) { PyErr_NoMemory(); return false; }
     }
 
     if (!self->lock_inited) {
@@ -418,13 +522,15 @@ ensure_state(DiskCache *self) {
     }
 
     if (self->cache_file_fd < 0) {
-        self->cache_file_fd = open_cache_file(self->cache_dir);
+        bool opened_securely;
+        self->cache_file_fd = open_cache_file(self->cache_dir, &opened_securely);
         if (self->cache_file_fd < 0) {
             PyErr_SetFromErrnoWithFilename(PyExc_OSError, self->cache_dir);
             return false;
         }
+        self->needs_encryption = !opened_securely;
     }
-
+    vt_init(&self->map); vt_init(&self->holes.pos_map); vt_init(&self->holes.size_map); vt_init(&self->holes.end_pos_map);
     self->fully_initialized = true;
     return true;
 }
@@ -442,8 +548,8 @@ dealloc(DiskCache* self) {
         pthread_join(self->write_thread, NULL);
         self->thread_started = false;
     }
-    if (self->currently_writing.hash_key) {
-        free(self->currently_writing.hash_key); self->currently_writing.hash_key = NULL;
+    if (self->currently_writing.key.hash_key) {
+        free(self->currently_writing.key.hash_key); self->currently_writing.key.hash_key = NULL;
     }
     if (self->lock_inited) {
         pthread_mutex_destroy(&self->lock);
@@ -453,33 +559,20 @@ dealloc(DiskCache* self) {
         free_loop_data(&self->loop_data);
         self->loop_data_inited = false;
     }
-    if (self->entries) {
-        CacheEntry *tmp, *s;
-        HASH_ITER(hh, self->entries, s, tmp) {
-            HASH_DEL(self->entries, s);
-            free_cache_entry(s); s = NULL;
-        }
-        self->entries = NULL;
-    }
+    vt_cleanup(&self->map); cleanup_holes(&self->holes);
     if (self->cache_file_fd > -1) {
         safe_close(self->cache_file_fd, __FILE__, __LINE__);
         self->cache_file_fd = -1;
     }
-    free(self->holes.items); zero_at_ptr(&self->holes);
-    if (self->currently_writing.data) free(self->currently_writing.data);
+    if (self->currently_writing.val.data) free(self->currently_writing.val.data);
     free(self->cache_dir); self->cache_dir = NULL;
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
-static CacheEntry*
-create_cache_entry(const void *key, const size_t key_sz) {
-    CacheEntry *s = calloc(1, sizeof(CacheEntry));
-    if (!s) return (CacheEntry*)PyErr_NoMemory();
-    if (!secure_random_bytes(s->encryption_key, sizeof(s->encryption_key))) { free(s); PyErr_SetFromErrno(PyExc_OSError); return NULL; }
-    s->hash_key = malloc(key_sz);
-    if (!s->hash_key) { free(s); PyErr_NoMemory(); return NULL; }
-    s->hash_keylen = key_sz;
-    memcpy(s->hash_key, key, key_sz);
+static CacheValue*
+create_cache_entry(void) {
+    CacheValue *s = calloc(1, sizeof(CacheValue));
+    if (!s) return (CacheValue*)PyErr_NoMemory();
     s->pos_in_cache_file = -2;
     return s;
 }
@@ -489,17 +582,22 @@ add_to_disk_cache(PyObject *self_, const void *key, size_t key_sz, const void *d
     DiskCache *self = (DiskCache*)self_;
     if (!ensure_state(self)) return false;
     if (key_sz > MAX_KEY_SIZE) { PyErr_SetString(PyExc_KeyError, "cache key is too long"); return false; }
-    CacheEntry *s = NULL;
     RAII_ALLOC(uint8_t, copied_data, malloc(data_sz));
     if (!copied_data) { PyErr_NoMemory(); return false; }
     memcpy(copied_data, data, data_sz);
+    CacheKey k = {.hash_key=(void*)key, .hash_keylen=key_sz};
 
     mutex(lock);
-    HASH_FIND(hh, self->entries, key, key_sz, s);
-    if (s == NULL) {
-        if (!(s = create_cache_entry(key, key_sz))) goto end;
-        HASH_ADD_KEYPTR(hh, self->entries, s->hash_key, s->hash_keylen, s);
+    cache_map_itr i = vt_get(&self->map, k);
+    CacheValue *s;
+    if (vt_is_end(i)) {
+        k.hash_key = malloc(key_sz);
+        if (!k.hash_key) { PyErr_NoMemory(); goto end; }
+        memcpy(k.hash_key, key, key_sz);
+        if (!(s = create_cache_entry())) goto end;
+        if (vt_is_end(vt_insert(&self->map, k, s))) { PyErr_NoMemory(); goto end; }
     } else {
+        s = i.data->val;
         remove_from_disk(self, s);
         self->total_size -= MIN(self->total_size, s->data_sz);
         if (s->data) free(s->data);
@@ -518,39 +616,36 @@ remove_from_disk_cache(PyObject *self_, const void *key, size_t key_sz) {
     DiskCache *self = (DiskCache*)self_;
     if (!ensure_state(self)) return false;
     if (key_sz > MAX_KEY_SIZE) { PyErr_SetString(PyExc_KeyError, "cache key is too long"); return false; }
-    CacheEntry *s = NULL;
+    CacheValue *s = NULL;
+    CacheKey k = {.hash_key=(void*)key, .hash_keylen=key_sz};
     bool removed = false;
 
     mutex(lock);
-    HASH_FIND(hh, self->entries, key, key_sz, s);
-    if (s) {
+    cache_map_itr i = vt_get(&self->map, k);
+    if (!vt_is_end(i)) {
         removed = true;
-        HASH_DEL(self->entries, s);
+        s = i.data->val;
         remove_from_disk(self, s);
         self->total_size = (self->total_size > s->data_sz) ? self->total_size - s->data_sz : 0;
-        free_cache_entry(s);
+        vt_erase_itr(&self->map, i);
     }
     mutex(unlock);
     wakeup_write_loop(self);
     return removed;
 }
 
-void
+static int
 clear_disk_cache(PyObject *self_) {
+    // This is currently only used in testing
     DiskCache *self = (DiskCache*)self_;
-    if (!ensure_state(self)) return;
-    CacheEntry *s, *tmp;
+    if (!ensure_state(self)) return 0;
+    int saved_errno = 0;
+    disk_cache_wait_for_write(self_, 0);
     mutex(lock);
-    HASH_ITER(hh, self->entries, s, tmp) {
-        HASH_DEL(self->entries, s);
-        free_cache_entry(s);
-    }
-    self->total_size = 0;
-    self->holes.count = 0;
-    self->holes.largest_hole_size = 0;
-    if (self->cache_file_fd > -1) add_hole(self, 0, size_of_cache_file(self));
+    saved_errno = clear_disk_cache_with_lock_held(self);
     mutex(unlock);
     wakeup_write_loop(self);
+    return saved_errno;
 }
 
 static void
@@ -577,7 +672,7 @@ read_from_cache_file(const DiskCache *self, off_t pos, size_t sz, void *dest) {
 }
 
 static void
-read_from_cache_entry(const DiskCache *self, const CacheEntry *s, void *dest) {
+read_from_cache_entry(const DiskCache *self, const CacheValue *s, void *dest) {
     size_t sz = s->data_sz;
     off_t pos = s->pos_in_cache_file;
     if (pos < 0) {
@@ -593,22 +688,23 @@ read_from_disk_cache(PyObject *self_, const void *key, size_t key_sz, void*(allo
     void *data = NULL;
     if (!ensure_state(self)) return data;
     if (key_sz > MAX_KEY_SIZE) { PyErr_SetString(PyExc_KeyError, "cache key is too long"); return data; }
+    CacheKey k = {.hash_key=(void*)key, .hash_keylen=key_sz};
 
     mutex(lock);
-    CacheEntry *s = NULL;
-    HASH_FIND(hh, self->entries, key, key_sz, s);
-    if (!s) { PyErr_SetString(PyExc_KeyError, "No cached entry with specified key found"); goto end; }
+    cache_map_itr i = vt_get(&self->map, k);
+    if (vt_is_end(i)) { PyErr_SetString(PyExc_KeyError, "No cached entry with specified key found"); goto end; }
+    CacheValue *s = i.data->val;
     data = allocator(allocator_data, s->data_sz);
     if (!data) { PyErr_NoMemory(); goto end; }
 
     if (s->data) { memcpy(data, s->data, s->data_sz); }
-    else if (self->currently_writing.data && self->currently_writing.hash_key && self->currently_writing.hash_keylen == key_sz && memcmp(self->currently_writing.hash_key, key, key_sz) == 0) {
-        memcpy(data, self->currently_writing.data, s->data_sz);
-        xor_data64(s->encryption_key, data, s->data_sz);
+    else if (self->currently_writing.val.data && self->currently_writing.key.hash_key && keys_are_equal(self->currently_writing.key, k)) {
+        memcpy(data, self->currently_writing.val.data, s->data_sz);
+        if (s->uses_encryption) xor_data64(s->encryption_key, data, s->data_sz);
     }
     else {
         read_from_cache_entry(self, s, data);
-        xor_data64(s->encryption_key, data, s->data_sz);
+        if (s->uses_encryption) xor_data64(s->encryption_key, data, s->data_sz);
     }
     if (store_in_ram && !s->data && s->data_sz) {
         void *copy = malloc(s->data_sz);
@@ -627,9 +723,9 @@ disk_cache_clear_from_ram(PyObject *self_, bool(matches)(void*, void *key, unsig
     size_t ans = 0;
     if (!ensure_state(self)) return ans;
     mutex(lock);
-    CacheEntry *s, *tmp;
-    HASH_ITER(hh, self->entries, s, tmp) {
-        if (s->written_to_disk && s->data && matches(data, s->hash_key, s->hash_keylen)) {
+    cache_map_for_loop(i) {
+        CacheValue *s = i.data->val;
+        if (s->written_to_disk && s->data && matches(data, i.data->key.hash_key, i.data->key.hash_keylen)) {
             free(s->data); s->data = NULL;
             ans++;
         }
@@ -646,9 +742,8 @@ disk_cache_wait_for_write(PyObject *self_, monotonic_t timeout) {
     while (!timeout || monotonic() <= end_at) {
         bool pending = false;
         mutex(lock);
-        CacheEntry *s, *tmp;
-        HASH_ITER(hh, self->entries, s, tmp) {
-            if (!s->written_to_disk) {
+        cache_map_for_loop(i) {
+            if (!i.data->val->written_to_disk) {
                 pending = true;
                 break;
             }
@@ -671,9 +766,8 @@ disk_cache_num_cached_in_ram(PyObject *self_) {
     unsigned long ans = 0;
     if (ensure_state(self)) {
         mutex(lock);
-        CacheEntry *tmp, *s;
-        HASH_ITER(hh, self->entries, s, tmp) {
-            if (s->written_to_disk && s->data) ans++;
+        cache_map_for_loop(i) {
+            if (i.data->val->written_to_disk && i.data->val->data) ans++;
         }
         mutex(unlock);
     }
@@ -681,6 +775,7 @@ disk_cache_num_cached_in_ram(PyObject *self_) {
 }
 
 
+// The Python interface used only for testing {{{
 #define PYWRAP(name) static PyObject* py##name(DiskCache *self, PyObject *args)
 #define PA(fmt, ...) if (!PyArg_ParseTuple(args, fmt, __VA_ARGS__)) return NULL;
 PYWRAP(ensure_state) {
@@ -689,17 +784,6 @@ PYWRAP(ensure_state) {
     Py_RETURN_NONE;
 }
 
-PYWRAP(read_from_cache_file) {
-    Py_ssize_t pos = 0, sz = -1;
-    PA("|nn", &pos, &sz);
-    if (sz < 0) sz = size_of_cache_file(self);
-    PyObject *ans = PyBytes_FromStringAndSize(NULL, sz);
-    if (ans) {
-        read_from_cache_file(self, pos, sz, PyBytes_AS_STRING(ans));
-    }
-    return ans;
-}
-
 static PyObject*
 wait_for_write(PyObject *self, PyObject *args) {
     double timeout = 0;
@@ -709,17 +793,40 @@ wait_for_write(PyObject *self, PyObject *args) {
 }
 
 static PyObject*
-size_on_disk(PyObject *self, PyObject *args UNUSED) {
-    unsigned long long ans = disk_cache_size_on_disk(self);
+end_of_data_offset(PyObject *self_, PyObject *args UNUSED) {
+    // Only used for testing
+    DiskCache *self = (DiskCache*)self_;
+    unsigned long long ans = 0;
+    mutex(lock);
+    if (self->cache_file_fd > -1) ans = MAX(0, self->end_of_data_offset);
+    mutex(unlock);
     return PyLong_FromUnsignedLongLong(ans);
 }
 
 static PyObject*
 clear(PyObject *self, PyObject *args UNUSED) {
-    clear_disk_cache(self);
+    int saved_errno = clear_disk_cache(self);
+    if (saved_errno) return PyErr_SetFromErrno(PyExc_OSError);
     Py_RETURN_NONE;
 }
 
+static PyObject*
+holes(PyObject *self_, PyObject *args UNUSED) {
+    DiskCache *self = (DiskCache*)self_;
+    mutex(lock);
+    RAII_PyObject(ans, PyFrozenSet_New(NULL));
+    if (ans) {
+        hole_pos_map_for_loop(i) {
+            RAII_PyObject(t, Py_BuildValue("LL", (long long)i.data->key, (long long)i.data->val));
+            if (!t || PySet_Add(ans, t) != 0) break;
+        }
+    }
+    mutex(unlock);
+    if (PyErr_Occurred()) return NULL;
+    Py_INCREF(ans);
+    return ans;
+}
+
 
 static PyObject*
 add(PyObject *self, PyObject *args) {
@@ -794,15 +901,15 @@ num_cached_in_ram(PyObject *self, PyObject *args UNUSED) {
 #define MW(name, arg_type) {#name, (PyCFunction)py##name, arg_type, NULL}
 static PyMethodDef methods[] = {
     MW(ensure_state, METH_NOARGS),
-    MW(read_from_cache_file, METH_VARARGS),
     {"add", add, METH_VARARGS, NULL},
     {"remove", pyremove, METH_VARARGS, NULL},
     {"remove_from_ram", remove_from_ram, METH_O, NULL},
     {"num_cached_in_ram", num_cached_in_ram, METH_NOARGS, NULL},
     {"get", get, METH_VARARGS, NULL},
     {"wait_for_write", wait_for_write, METH_VARARGS, NULL},
-    {"size_on_disk", size_on_disk, METH_NOARGS, NULL},
+    {"end_of_data_offset", end_of_data_offset, METH_NOARGS, NULL},
     {"clear", clear, METH_NOARGS, NULL},
+    {"holes", holes, METH_NOARGS, NULL},
 
     {NULL}  /* Sentinel */
 };
@@ -810,6 +917,7 @@ static PyMethodDef methods[] = {
 static PyMemberDef members[] = {
     {"total_size", T_ULONGLONG, offsetof(DiskCache, total_size), READONLY, "total_size"},
     {"small_hole_threshold", T_PYSSIZET, offsetof(DiskCache, small_hole_threshold), 0, "small_hole_threshold"},
+    {"defrag_factor", T_UINT, offsetof(DiskCache, defrag_factor), 0, "defrag_factor"},
     {NULL},
 };
 
@@ -828,3 +936,4 @@ PyTypeObject DiskCache_Type = {
 
 INIT_TYPE(DiskCache)
 PyObject* create_disk_cache(void) { return new_diskcache_object(&DiskCache_Type, NULL, NULL); }
+// }}}
