@@ -52,9 +52,10 @@ ssize_t create_vao(void);
 size_t add_buffer_to_vao(ssize_t vao_idx, GLenum usage);
 void add_attribute_to_vao(int p, ssize_t vao_idx, const char *name, GLint size, GLenum data_type, GLsizei stride, void *offset, GLuint divisor);
 ssize_t alloc_vao_buffer(ssize_t vao_idx, GLsizeiptr size, size_t bufnum, GLenum usage);
-void* alloc_and_map_vao_buffer(ssize_t vao_idx, GLsizeiptr size, size_t bufnum, GLenum usage, GLenum access);
+void* alloc_and_map_vao_buffer(ssize_t vao_idx, GLsizeiptr size, size_t bufnum, bool frequently_updated);
 void unmap_vao_buffer(ssize_t vao_idx, size_t bufnum);
 void* map_vao_buffer(ssize_t vao_idx, size_t bufnum, GLenum access);
+void* map_vao_buffer_for_write_only(ssize_t vao_idx, size_t bufnum, int offset, unsigned size);
 void bind_program(int program);
 void bind_vertex_array(ssize_t vao_idx);
 void bind_vao_uniform_buffer(ssize_t vao_idx, size_t bufnum, GLuint block_index);
