@@ -22,6 +22,22 @@ cwd_of_process(PyObject *self UNUSED, PyObject *pid_) {
     return PyUnicode_FromString(vpi.pvi_cdir.vip_path);
 }
 
+static PyObject*
+abspath_of_process(PyObject *self UNUSED, PyObject *pid_) {
+    if (!PyLong_Check(pid_)) { PyErr_SetString(PyExc_TypeError, "pid must be an int"); return NULL; }
+    pid_t pid = PyLong_AsLong(pid_);
+    if (pid < 0) { PyErr_SetString(PyExc_TypeError, "pid cannot be negative"); return NULL; }
+    char pathbuf[PROC_PIDPATHINFO_MAXSIZE];
+    int ret = proc_pidpath(pid, pathbuf, sizeof(pathbuf));
+    if (ret < 0) {
+        PyErr_SetFromErrno(PyExc_OSError); return NULL;
+    } else if (ret == 0) {
+        errno = EINVAL;
+        PyErr_SetFromErrno(PyExc_OSError); return NULL;
+    }
+    return PyUnicode_FromString(pathbuf);
+}
+
 // Read the maximum argument size for processes
 static int
 get_argmax(void) {
@@ -96,7 +112,7 @@ cmdline_of_process(PyObject *self UNUSED, PyObject *pid_) {
     if (sysctl(mib, 3, procargs, &argmax, NULL, 0) < 0) {
         // In case of zombie process or non-existent process we'll get EINVAL.
         if (errno == EINVAL)
-            PyErr_Format(PyExc_ValueError, "process with pid %ld either does not exist or is a zombie", pid);
+            PyErr_Format(PyExc_ValueError, "process with pid %ld either does not exist or is a zombie or you dont have permission", pid);
         else
             PyErr_SetFromErrno(PyExc_OSError);
         goto error;
@@ -193,7 +209,7 @@ environ_of_process(PyObject *self UNUSED, PyObject *pid_) {
         // In case of zombie process or a non-existent process we'll get EINVAL
         // to NSP and _psosx.py will translate it to ZP.
         if (errno == EINVAL)
-            PyErr_Format(PyExc_ValueError, "process with pid %ld either does not exist or is a zombie", pid);
+            PyErr_Format(PyExc_ValueError, "process with pid %ld either does not exist or is a zombie or you dont have permission", pid);
         else
             PyErr_SetFromErrno(PyExc_OSError);
         goto error;
@@ -272,6 +288,7 @@ error:
 
 static PyMethodDef module_methods[] = {
     {"cwd_of_process", (PyCFunction)cwd_of_process, METH_O, ""},
+    {"abspath_of_process", (PyCFunction)abspath_of_process, METH_O, ""},
     {"cmdline_of_process", (PyCFunction)cmdline_of_process, METH_O, ""},
     {"environ_of_process", (PyCFunction)environ_of_process, METH_O, ""},
     {"get_all_processes", (PyCFunction)get_all_processes, METH_NOARGS, ""},
