@@ -1,9 +1,9 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 import re
 from contextlib import suppress
-from typing import Optional
+
 from .fast_data_types import Color
 
 
@@ -25,21 +25,32 @@ def parse_single_color(c: str) -> int:
     return int(c[:2], 16)
 
 
-def parse_sharp(spec: str) -> Optional[Color]:
+def parse_sharp(spec: str) -> Color | None:
     if len(spec) in (3, 6, 9, 12):
         part_len = len(spec) // 3
-        colors = re.findall(r'[a-fA-F0-9]{%d}' % part_len, spec)
+        colors = re.findall(fr'[a-fA-F0-9]{{{part_len}}}', spec)
         return Color(*map(parse_single_color, colors))
     return None
 
 
-def parse_rgb(spec: str) -> Optional[Color]:
+def parse_rgb(spec: str) -> Color | None:
     colors = spec.split('/')
     if len(colors) == 3:
         return Color(*map(parse_single_color, colors))
     return None
 
 
+def parse_single_intensity(x: str) -> int:
+    return int(max(0, min(abs(float(x)), 1)) * 255)
+
+
+def parse_rgbi(spec: str) -> Color | None:
+    colors = spec.split('/')
+    if len(colors) == 3:
+        return Color(*map(parse_single_intensity, colors))
+    return None
+
+
 def color_from_int(x: int) -> Color:
     return Color((x >> 16) & 255, (x >> 8) & 255, x & 255)
 
@@ -56,20 +67,24 @@ def color_as_sgr(x: Color) -> str:
     return x.as_sgr
 
 
-def to_color(raw: str, validate: bool = False) -> Optional[Color]:
+def to_color(raw: str, validate: bool = False) -> Color | None:
     # See man XParseColor
     x = raw.strip().lower()
     ans = color_names.get(x)
     if ans is not None:
         return ans
-    val: Optional[Color] = None
+    val: Color | None = None
     with suppress(Exception):
         if raw.startswith('#'):
             val = parse_sharp(raw[1:])
-        elif raw[:4].lower() == 'rgb:':
-            val = parse_rgb(raw[4:])
+        else:
+            k, sep, v = raw.partition(':')
+            if k == 'rgb':
+                val = parse_rgb(v)
+            elif k == 'rgbi':
+                val = parse_rgbi(v)
     if val is None and validate:
-        raise ValueError(f'Invalid color name: {raw}')
+        raise ValueError(f'Invalid color name: {raw!r}')
     return val
 
 
@@ -829,27 +844,3 @@ color_names = {
  'yellow4': Color(139, 139, 0),
  'yellowgreen': Color(154, 205, 50)}
 # END_DATA_SECTION }}}
-
-if __name__ == '__main__':
-    # Read RGB color table from specified rgb.txt file
-    import pprint
-    import sys
-    data = {}
-    with open(sys.argv[-1]) as f:
-        for line in f:
-            line = line.strip()
-            if not line or line.startswith('!'):
-                continue
-            parts = line.split()
-            r, g, b = map(int, parts[:3])
-            name = ' '.join(parts[3:]).lower()
-            data[name] = data[name.replace(' ', '')] = r, g, b
-    formatted_data = pprint.pformat(data).replace('{', '{\n ').replace('(', 'Color(')
-    with open(__file__, 'r+') as src:
-        raw = src.read()
-        raw = re.sub(
-            r'^# BEGIN_DATA_SECTION {{{$.*^# END_DATA_SECTION }}}',
-            '# BEGIN_DATA_SECTION {{{\ncolor_names = %s\n# END_DATA_SECTION }}}' % formatted_data,
-            raw, flags=re.DOTALL | re.MULTILINE
-        )
-        src.seek(0), src.truncate(), src.write(raw)
