@@ -267,7 +267,7 @@ class TestLayout(BaseTest):
         windows[1].set_geometry(WindowGeometry(11, 0, 20, 10, 0, 0))
         windows[2].set_geometry(WindowGeometry(11, 11, 15, 20, 0, 0))
         windows[3].set_geometry(WindowGeometry(16, 11, 20, 20, 0, 0))
-        self.ae(q.neighbors_for_window(windows[0], all_windows), {'left': [], 'right': [2, 3], 'top': [], 'bottom': []})
-        self.ae(q.neighbors_for_window(windows[1], all_windows), {'left': [1], 'right': [], 'top': [], 'bottom': [3, 4]})
-        self.ae(q.neighbors_for_window(windows[2], all_windows), {'left': [1], 'right': [4], 'top': [2], 'bottom': []})
-        self.ae(q.neighbors_for_window(windows[3], all_windows), {'left': [3], 'right': [], 'top': [2], 'bottom': []})
+        self.ae(q.neighbors_for_window(windows[0], all_windows), {'right': [2, 3]})
+        self.ae(q.neighbors_for_window(windows[1], all_windows), {'left': [1], 'bottom': [3, 4]})
+        self.ae(q.neighbors_for_window(windows[2], all_windows), {'left': [1], 'right': [4], 'top': [2]})
+        self.ae(q.neighbors_for_window(windows[3], all_windows), {'left': [3], 'top': [2]})
