@@ -769,8 +769,9 @@ class TestParser(BaseTest):
         self.ae(c.wtcbuf, b'\033[?1;2$y')
         pb('\033[2;4r', ('screen_set_margins', 2, 4))
         c.clear()
-        pb('\033[14t', ('screen_report_size', 14))
-        self.ae(c.wtcbuf, b'\033[4;100;50t')
+        pb('\033[14t', ('screen_report_size', 14, 0))
+        pb('\033[14;2t', ('screen_report_size', 14, 2))
+        self.ae(c.wtcbuf, b'\033[4;100;50t\033[4;100;50t')
         self.ae(s.margin_top, 1), self.ae(s.margin_bottom, 3)
         pb('\033[r', ('screen_set_margins', 0, 0))
         self.ae(s.margin_top, 0), self.ae(s.margin_bottom, 4)
