@@ -3,14 +3,13 @@
 package completion
 
 import (
-	"bufio"
-	"bytes"
 	"fmt"
-	"os/exec"
 	"strings"
 
-	"kitty/tools/cli"
-	"kitty/tools/utils"
+	kitty_constants "github.com/kovidgoyal/kitty"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/themes"
+	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 var _ = fmt.Print
@@ -18,9 +17,11 @@ var _ = fmt.Print
 func complete_kitty_override(completions *cli.Completions, word string, arg_num int) {
 	mg := completions.AddMatchGroup("Config directives")
 	mg.NoTrailingSpace = true
-	for _, q := range kitty_option_names_for_completion {
-		if strings.HasPrefix(q, word) {
-			mg.AddMatch(q + "=")
+	scanner := utils.NewLineScanner(kitty_constants.OptionNames)
+	for scanner.Scan() {
+		line := strings.TrimSpace(scanner.Text())
+		if strings.HasPrefix(line, word) {
+			mg.AddMatch(line + "=")
 		}
 	}
 }
@@ -66,20 +67,7 @@ func complete_plus_open(completions *cli.Completions, word string, arg_num int)
 }
 
 func complete_themes(completions *cli.Completions, word string, arg_num int) {
-	kitty, err := utils.KittyExe()
-	if err == nil {
-		out, err := exec.Command(kitty, "+runpy", "from kittens.themes.collection import *; print_theme_names()").Output()
-		if err == nil {
-			mg := completions.AddMatchGroup("Themes")
-			scanner := bufio.NewScanner(bytes.NewReader(out))
-			for scanner.Scan() {
-				theme_name := strings.TrimSpace(scanner.Text())
-				if theme_name != "" && strings.HasPrefix(theme_name, word) {
-					mg.AddMatch(theme_name)
-				}
-			}
-		}
-	}
+	themes.CompleteThemes(completions, word, arg_num)
 }
 
 func EntryPoint(tool_root *cli.Command) {
