@@ -10,11 +10,11 @@ import (
 	"sync"
 	"sync/atomic"
 
+	"github.com/kovidgoyal/go-shm"
 	"github.com/kovidgoyal/kitty/tools/tui"
 	"github.com/kovidgoyal/kitty/tools/tui/loop"
 	"github.com/kovidgoyal/kitty/tools/utils"
 	"github.com/kovidgoyal/kitty/tools/utils/images"
-	"github.com/kovidgoyal/kitty/tools/utils/shm"
 )
 
 var _ = fmt.Print
@@ -336,7 +336,8 @@ func transmit_by_escape_code(lp *loop.Loop, image_id uint32, temp_file_map map[u
 	atomic := lp.IsAtomicUpdateActive()
 	lp.EndAtomicUpdate()
 	gc.SetTransmission(GRT_transmission_direct)
-	_ = gc.WriteWithPayloadToLoop(lp, frame.Data())
+	_, _, _, data := frame.Data()
+	_ = gc.WriteWithPayloadToLoop(lp, data)
 	if atomic {
 		lp.StartAtomicUpdate()
 	}
@@ -362,7 +363,8 @@ func transmit_by_file(lp *loop.Loop, image_id uint32, temp_file_map map[uint32]*
 	}
 	defer f.Close()
 	temp_file_map[image_id] = &temp_resource{path: f.Name()}
-	_, err = f.Write(frame.Data())
+	_, _, _, data := frame.Data()
+	_, err = f.Write(data)
 	if err != nil {
 		transmit_by_escape_code(lp, image_id, temp_file_map, frame, gc)
 		return
@@ -403,6 +405,9 @@ func (self *ImageCollection) transmit_rendering(lp *loop.Loop, r *rendering) {
 			if frame.Compose_onto > 0 {
 				gc.SetOverlaidFrame(uint64(frame.Compose_onto))
 			}
+			if frame.Replace {
+				gc.SetCompositionMode(Overwrite)
+			}
 			gc.SetLeftEdge(uint64(frame.Left)).SetTopEdge(uint64(frame.Top))
 		}
 		transmit(lp, r.image_id, self.temp_file_map, frame, gc)
