@@ -49,7 +49,7 @@ func (self *WCWidthIterator) handle_csi(csi []byte) error {
 		num_string := utils.UnsafeBytesToString(csi[:len(csi)-1])
 		n, err := strconv.Atoi(num_string)
 		if err == nil && n > 0 {
-			for i := 0; i < n; i++ {
+			for range n {
 				err = self.handle_rune(self.prev_ch)
 				if err != nil {
 					return err
